import android.content.Context
import android.os.Build
import android.util.Log
import androidx.annotation.RequiresApi
import com.google.android.gms.location.DetectedActivity
import com.google.gson.Gson
import com.google.gson.reflect.TypeToken
import com.reactnativeactivityrecognitionwrapper.types.ActivityObj
import com.reactnativeactivityrecognitionwrapper.utils.CacheManager
import com.reactnativeactivityrecognitionwrapper.utils.Constants
import java.lang.reflect.Type
import java.time.Instant
import java.time.LocalDateTime
import java.time.ZoneId
import java.time.ZonedDateTime
import java.time.format.DateTimeFormatter.ofPattern
import java.util.*

class ActivityHelper {
  val gson: Gson
  var cacheMan: CacheManager

  constructor(context: Context) {
    cacheMan = CacheManager(context)
    gson = Gson()
  }

  @RequiresApi(Build.VERSION_CODES.O)
  fun appendActivityToCache(activity: ActivityObj) {
    val json = cacheMan.getString(Constants.ACTIVITY_UPDATES)
    var activityList = ArrayList<ActivityObj>()
    if (json != null) {
      val type: Type = object : TypeToken<ArrayList<ActivityObj?>?>() {}.type
      activityList = gson.fromJson<Any>(json, type) as ArrayList<ActivityObj>
    }
    activityList.add(activity)
    var df = ofPattern(Constants.DATETIME_FORMAT).withZone(ZoneId.systemDefault())
    var currentDateTime = ZonedDateTime.now().toLocalDateTime()
    println("DATE: " + df.format(Instant.now()))
    activityList.removeAll { it.CONFIDENCE < 20 }
    activityList.removeAll { it.TYPE == "UNKNOWN" }
    activityList.removeAll {
      df.format(Instant.now()) < currentDateTime.minusSeconds(120).toString()
    }

    cacheMan.setString(Constants.ACTIVITY_UPDATES, gson.toJson(activityList))
    println(activityList)
  }

  @RequiresApi(Build.VERSION_CODES.O)
  fun resampleUserActivities() {
    val json = cacheMan.getString(Constants.ACTIVITY_UPDATES)
    var activityList: ArrayList<ActivityObj>
    if (json != null) {
      val type: Type = object : TypeToken<ArrayList<ActivityObj?>?>() {}.type
      activityList = gson.fromJson<Any>(json, type) as ArrayList<ActivityObj>
      if (activityList.size > 0) {
        var latestActivity = activityList[activityList.size - 1]
        var df = ofPattern(Constants.DATETIME_FORMAT).withZone(ZoneId.systemDefault())
        var currentDateTime = LocalDateTime.now()
        Log.i("ACTIVITYHELPER", currentDateTime.toString())
        Log.i("ACTIVITYHELPER", latestActivity.TIMESTAMP)
        Log.i("ACTIVITYHELPER", activityList.toString())

        if (LocalDateTime.parse(latestActivity.TIMESTAMP, df) < currentDateTime.minusSeconds(5)) {
          latestActivity.TIMESTAMP = LocalDateTime
            .parse(latestActivity.TIMESTAMP, df).plusSeconds(5).toString()
          activityList.add(latestActivity)
          cacheMan.setString(Constants.ACTIVITY_UPDATES, gson.toJson(activityList))
        }
        Log.i("ACTIVITYHELPER", activityList.toString())
        determineMostProbableActivity(activityList)
      }
    } else {
      Log.i("ACTIVITYHELPER", "No activities in cache - cannot determine user activity state")
    }
  }

  fun determineMostProbableActivity(activitiesList: ArrayList<ActivityObj>) {
//    var activities = (activitiesList.map { it.TYPE }).distinct()
//    var activitiesAndConfidences: Any =
//      activities.map { activity ->
//        activitiesList.filter {
//          it.TYPE == activity
//        }
//      }
//    println(activitiesAndConfidences)
  }

  fun getActivityObject(type: String, confidence: Int, timestamp: String): ActivityObj {
    return ActivityObj(timestamp, type, confidence)
  }

  fun convertActivityTypeIntToString(type: Int): String {
    var label = "UNKNOWN"

    when (type) {
      DetectedActivity.IN_VEHICLE -> {
        label = "IN_VEHICLE"

      }
      DetectedActivity.ON_BICYCLE -> {
        label = "CYCLING"
      }
      DetectedActivity.ON_FOOT -> {
        label = "ON_FOOT"
      }
      DetectedActivity.RUNNING -> {
        label = "RUNNING"
      }
      DetectedActivity.STILL -> {
        label = "STILL"
      }
      DetectedActivity.TILTING -> {
        label = "TILTING"
      }
      DetectedActivity.WALKING -> {
        label = "WALKING"
      }
      DetectedActivity.UNKNOWN -> {
        label = "UNKNOWN"
      }
    }
    return label


//    prepareActivityUpdateBroadcastAndSend(timestamp, label, confidence.toString())
  }
}
