import android.content.Context
import java.io.File
import java.io.FileOutputStream
import java.io.IOException
import java.io.OutputStreamWriter
import java.text.SimpleDateFormat
import java.util.Date
import java.util.Locale

class CsvFileHelper(private val context: Context) {

  private val csvFiles: MutableMap<Int, File> = mutableMapOf()
  private val outputStreamWriters: MutableMap<Int, OutputStreamWriter> = mutableMapOf()

  fun createCsvFile(sensorName: String, sensorType: Int, folderNames:List<String>) {
    var folder = context.filesDir
    folderNames.forEach { folderName ->
      folder = File(folder, folderName)
      if (!folder.exists()) {
        folder.mkdir()
      }
    }
    val fileName = "$sensorName.csv"
    val file = File(folder, fileName)
    val outputStreamWriter = OutputStreamWriter(FileOutputStream(file, true))

    csvFiles[sensorType] = file
    outputStreamWriters[sensorType] = outputStreamWriter
  }

  fun writeCsvLine(sensorType: Int, csvLine: String) {
    outputStreamWriters[sensorType]?.apply {
      write(csvLine)
      flush()
    }
  }

  fun closeCsvFiles() {
    outputStreamWriters.values.forEach {
      it.close()
    }
  }
}
