import android.annotation.SuppressLint
import android.content.Context
import android.location.Criteria
import android.location.Location
import android.location.LocationListener
import android.location.LocationManager
import android.os.Build
import android.os.Bundle
import android.util.Log
import androidx.annotation.RequiresApi
import com.reactnativeactivityrecognitionwrapper.types.LocationObj
import java.time.Instant
import java.time.ZoneId
import java.time.format.DateTimeFormatter.ofPattern


object LocationHelper {

  var LOCATION_REFRESH_DISTANCE =
    500 // 500 meters. The Minimum Distance to be changed to get location update
  private lateinit var mLocationManager: LocationManager
  private lateinit var locationListener: LocationListener

  @SuppressLint("MissingPermission")
  fun startListeningUserLocation(
    context: Context,
    myListener: MyLocationListener,
    distanceFilter: Int,
    accuracy: Int
  ) {
    val criteria = Criteria()
    if (accuracy > 20) {
      criteria.accuracy = Criteria.ACCURACY_COARSE
      criteria.powerRequirement = Criteria.POWER_LOW
      criteria.isCostAllowed = false

      Log.i("LOCATION_HELPER", "CHANGED LOCATION ACCURACY TO ACCURACY_COARSE")

    } else {
      criteria.accuracy = Criteria.ACCURACY_FINE
      criteria.powerRequirement = Criteria.POWER_HIGH
      criteria.isCostAllowed = true
      Log.i("LOCATION_HELPER", "CHANGED LOCATION ACCURACY TO ACCURACY_FINE AND INCREASED POWER USAGE")
    }

    mLocationManager = context.getSystemService(Context.LOCATION_SERVICE) as LocationManager
    val provider: String? = mLocationManager.getBestProvider(criteria, true)

    //ACCURACY IS NOT BEING USED ON ANDROID BECAUSE OF THE FUSED PROVIDER OPTION
    //NEED TO BE ABLE TO SPECIFY ACCURACY IN FUTURE.
    LOCATION_REFRESH_DISTANCE = distanceFilter
    locationListener = object : LocationListener {
      override fun onLocationChanged(location: Location) {
        myListener.onLocationChanged(location) // calling listener to inform that updated location is available
      }

      override fun onProviderEnabled(provider: String) {}
      override fun onProviderDisabled(provider: String) {}
      override fun onStatusChanged(provider: String, status: Int, extras: Bundle) {}
    }
    if (provider != null) {
      Log.i(
        "LOCATION_HELPER",
        "STARTING LOCATION UPDATES WITH OBTAINED PROVIDER FOR REQUESTED ACCURACY"
      )
      mLocationManager.requestLocationUpdates(
        provider,
        3000,
        LOCATION_REFRESH_DISTANCE.toFloat(),
        locationListener
      )
    } else {
      Log.i(
        "LOCATION_HELPER",
        "COULD NOT OBTAIN PROVIDER FOR REQUESTED ACCURACY, DEFAULTING TO FUSED PROVIDER"
      )
      mLocationManager.requestLocationUpdates(
        LocationManager.FUSED_PROVIDER,
        3000,
        LOCATION_REFRESH_DISTANCE.toFloat(),
        locationListener
      )
    }

  }

  fun stopLocationUpdates() {
    if (this::mLocationManager.isInitialized) {
      mLocationManager.removeUpdates(locationListener)
    }
  }
//
//  /**Mimics the behaviour of the iOS CoreMotion API function called startMonitoringSignificantLocationChanges,
//   * the distance filter is set to 500m to save battery.
//   */
//  @SuppressLint("MissingPermission")
//  fun startListeningSignificantLocation(context: Context, myListener: MyLocationListener) {
//    mLocationManager= context.getSystemService(Context.LOCATION_SERVICE) as LocationManager
//    locationListener = object : LocationListener {
//      override fun onLocationChanged(location: Location) {
//        myListener.onLocationChanged(location) // calling listener to inform that updated location is available
//      }
//      override fun onProviderEnabled(provider: String) {}
//      override fun onProviderDisabled(provider: String) {}
//      override fun onStatusChanged(provider: String, status: Int, extras: Bundle) {}
//    }
//
//    mLocationManager.requestLocationUpdates(
//      LocationManager.FUSED_PROVIDER,
//      3000,
//      500.toFloat(),
//      locationListener
//    )
//  }


  @SuppressLint("MissingPermission")
  fun getLastKnownLocation(context: Context): Location? {
    val mLocationManager = context.getSystemService(Context.LOCATION_SERVICE) as LocationManager
    var loc = mLocationManager.getLastKnownLocation(LocationManager.FUSED_PROVIDER)
    if (loc != null) {
      Log.i("GETLASTKNOWNLOC", "Got location $loc")
      return loc
    } else {
      return null
    }
  }


  @RequiresApi(Build.VERSION_CODES.O)
  fun getTransformedLocationObject(location: Location): LocationObj {
    var timestamp = location.time
    return LocationObj(
      ofPattern("yyyy-MM-dd HH:mm:ss.SSS").withZone(ZoneId.systemDefault())
        .format(Instant.ofEpochMilli(timestamp)).toString(),
      location.latitude,
      location.longitude,
      location.accuracy,
      location.verticalAccuracyMeters,
      location.speedAccuracyMetersPerSecond,
      location.speed,
      location.altitude,
      location.bearing
    )
  }

}


interface MyLocationListener {
  fun onLocationChanged(location: Location?)
}
