package com.reactnativeactivityrecognitionwrapper.services

import android.app.IntentService
import android.content.Context
import android.content.Intent
import android.os.Build
import android.util.Log
import androidx.annotation.RequiresApi
import androidx.localbroadcastmanager.content.LocalBroadcastManager
import com.google.android.gms.location.*
import com.reactnativeactivityrecognitionwrapper.utils.Constants
import java.time.Instant
import java.time.ZoneId
import java.time.format.DateTimeFormatter
import java.util.*

@Suppress("DEPRECATION")
class ActivityDetectionIntentService :
  IntentService(ActivityDetectionIntentService::class.simpleName) {

  @RequiresApi(Build.VERSION_CODES.O)
  override fun onHandleIntent(intent: Intent?) {
    val result = ActivityRecognitionResult.extractResult(intent)
    result?.let {
      val detectedActivities: ArrayList<*> = it.probableActivities as ArrayList<*>
      Log.i("ActivityDetectServ", detectedActivities.toString())
      for (activity in detectedActivities) {

        broadcastActivity(activity!! as DetectedActivity)
      }
    }
  }


  @RequiresApi(Build.VERSION_CODES.O)
  private fun broadcastActivity(activity: DetectedActivity) {
    Log.i("DEBUG_ACTIVITY_BCAST", "This is activity: ${activity}")
    val intent = Intent(Constants.BROADCAST_DETECTED_ACTIVITY)
    intent.putExtra("type", activity.type)
    intent.putExtra("confidence", activity.confidence)
    intent.putExtra(
      "timestamp", DateTimeFormatter.ofPattern(Constants.DATETIME_FORMAT)
        .withZone(ZoneId.systemDefault())
        .format(Instant.now()).toString()
    )
    LocalBroadcastManager.getInstance(this).sendBroadcast(intent)
  }


}
