package com.reactnativeactivityrecognitionwrapper.services

import CsvFileHelper
import LocationHelper
import MyLocationListener
import SensorInfo
import SensorRequestItem
import android.app.*
import android.content.BroadcastReceiver
import android.content.Context
import android.content.Intent
import android.content.IntentFilter
import android.graphics.Color
import android.location.Location
import android.os.*
import android.util.Log
import android.widget.Toast
import androidx.annotation.RequiresApi
import androidx.localbroadcastmanager.content.LocalBroadcastManager
import com.google.android.gms.location.ActivityRecognitionClient
import com.reactnativeactivityrecognitionwrapper.R
import com.reactnativeactivityrecognitionwrapper.types.LocationObj
import com.reactnativeactivityrecognitionwrapper.utils.Constants
import java.time.ZonedDateTime
import java.util.*
import android.hardware.Sensor
import android.hardware.SensorEvent
import android.hardware.SensorEventListener
import android.hardware.SensorManager
import com.google.gson.Gson
import com.google.gson.reflect.TypeToken
import kotlin.collections.ArrayList

@Suppress("DEPRECATION", "UNREACHABLE_CODE")
class BackgroundDetectedActivitiesService : Service(), SensorEventListener {

  private lateinit var mIntentService: Intent
  private lateinit var mPendingIntent: PendingIntent
  private lateinit var mActivityRecognitionClient: ActivityRecognitionClient
  private var updateDuration: Int = 10000
  private var distanceFilter: Int = 10000
  private var accuracy: Int = 30
  private lateinit var sensorManager: SensorManager
  private lateinit var csvFileHelper: CsvFileHelper
  private lateinit var sensorInfoList: MutableList<SensorInfo>

  internal var mBinder: IBinder = LocalBinder()
  private var updatesRunning: Boolean = false
  var motionState: String = "STILL"
  var countdownTimerTicker = 0
  val handler = Handler(Looper.getMainLooper())
  lateinit var runnable:Runnable;

  inner class LocalBinder : Binder() {
    val serverInstance: BackgroundDetectedActivitiesService
      get() = this@BackgroundDetectedActivitiesService
  }

  private var running: Boolean = false

  private val sensorActivationReceiver = object : BroadcastReceiver() {
    override fun onReceive(context: Context, intent: Intent) {
      if (intent.action == "com.reactnativeactivityrecognitionwrapper.ACTION_ACTIVATE_SENSORS") {
        val sensorsRequestedList =
          intent.getSerializableExtra("sensorInfoList") as? List<SensorRequestItem> ?: emptyList()
        sensorInfoList = mutableListOf()
        Log.i("SENSOR_START","These are the sensors")

        Log.i("SENSOR_START",sensorsRequestedList.toString())
        sensorsRequestedList.forEach{
          print(it)
          var sensorInfoItem = SensorInfo.getSensorByName(it.sensorType)
          sensorInfoItem.samplingRate = it.samplingRate

          sensorInfoList.add(sensorInfoItem)
        }
        Log.i("SENSOR_START",sensorInfoList.toString())

        val folderStructureString = intent.getStringExtra("folderStructure") ?: "sensor_data"
        val folderStructure: List<String> = folderStructureString.split("/")

        activateSensors(folderStructure, sensorInfoList)
      }
    }
  }

  private val sensorDeactivationReceiver = object : BroadcastReceiver() {
    override fun onReceive(context: Context, intent: Intent) {
      if (intent.action == "com.reactnativeactivityrecognitionwrapper.ACTION_DEACTIVATE_SENSORS") {
        stopSensors()
      }
    }
  }

  @RequiresApi(Build.VERSION_CODES.O)
  override fun onCreate() {
    super.onCreate()
    LocalBroadcastManager.getInstance(this).registerReceiver(
      sensorActivationReceiver,
      IntentFilter("com.reactnativeactivityrecognitionwrapper.ACTION_ACTIVATE_SENSORS")
    )
    LocalBroadcastManager.getInstance(this).registerReceiver(
      sensorDeactivationReceiver,
      IntentFilter("com.reactnativeactivityrecognitionwrapper.ACTION_DEACTIVATE_SENSORS")
    )
    mActivityRecognitionClient = ActivityRecognitionClient(this)
    mIntentService = Intent(this, ActivityDetectionIntentService::class.java)
    mPendingIntent =
      PendingIntent.getService(this, 1, mIntentService, PendingIntent.FLAG_MUTABLE)
    val notificationChannel =
      createNotificationChannel("ActivityRecognitionService", "Activity Recognition Service")
    val notification: Notification = Notification.Builder(this, notificationChannel)
      .setContentTitle(getText(R.string.notification_title))
      .setContentText(getText(R.string.notification_message))
      .setContentIntent(mPendingIntent)
      .setSmallIcon(R.mipmap.breez_launcher)
      .setTicker("Tree planting service active.")
      .build()
    Log.i(TAG, "Created bg service")
//    activityHelper = ActivityHelper(applicationContext)
    startForeground(101, notification)
  }

  override fun onBind(intent: Intent): IBinder? {
    return mBinder
  }

  @RequiresApi(Build.VERSION_CODES.O)
  override fun onStartCommand(intent: Intent?, flags: Int, startId: Int): Int {
//    registerActivityReceiver()

    if (running) {
      LocationHelper.stopLocationUpdates()
      countdownTimerTicker = 0
      removeActivityUpdatesButtonHandler()
    }


    val notificationChannel =
      createNotificationChannel("ActivityRecognitionService", "Activity Recognition Service")
    val notification: Notification = Notification.Builder(this, notificationChannel)
      .setContentTitle(intent?.getStringExtra("NOTIFICATIONTITLE") ?: "Fail")
      .setContentText(intent?.getStringExtra("NOTIFICATIONDESCRIPTION") ?: "Fail").build()
    super.onStartCommand(intent, flags, startId)
    running = true
    var currentDateTime = ZonedDateTime.now().toLocalDateTime()
    println(currentDateTime)
    if (intent != null) {
      updateDuration = intent.getIntExtra("UPDATE_DURATION", 100000)
      accuracy = intent.getIntExtra("ACCURACY", 20)




      LocationHelper.startListeningUserLocation(
        this, object : MyLocationListener {
          override fun onLocationChanged(location: Location?) {
            mLocation = location
            mLocation?.let {
              Log.i("LOACTION RECEIVED", "Activity updates running: " + updatesRunning)
              if (!updatesRunning) {
                requestActivityUpdates()
                println("Starting motion updates for " + updateDuration + "ms")
                updatesRunning = true
                startActivityCountdownTimer(updateDuration)
              } else {
                Log.i("ACTIVITY COUNTDOWN", "RESET TIMER")
                countdownTimerTicker = 0
              }
              val location = LocationHelper.getTransformedLocationObject(it)
              broadcastLocation(location)
            }
          }
        },
        intent.getIntExtra("DISTANCE_FILTER", 100),
        accuracy
      )


      Log.i(TAG, "Started location tracking for activity updates service")

    } else {
      Log.i(TAG, "TRIED TO START BG SERVICE AGAIN BUT IT WAS DENIED")

    }
    return START_STICKY
  }

  fun startActivityCountdownTimer(updateDuration: Int) {

    runnable = object :  Runnable {
      override fun run() {
        countdownTimerTicker++
        if (countdownTimerTicker <= 50) {
          // Do something on each tick
          handler.postDelayed(this, 1000)
        } else {
          Log.i("ACTIVITY TIMER", "STOPPING ACTIVITY UPDATES")
          countdownTimerTicker = 0
          updatesRunning = false
          removeActivityUpdatesButtonHandler()
        }
      }
    }
    handler.post(runnable)
  }

  private fun broadcastLocation(location: LocationObj) {

    val intent = Intent(Constants.BROADCAST_DETECTED_LOCATION)
    intent.putExtra("locationObj", location)
    LocalBroadcastManager.getInstance(this).sendBroadcast(intent)
  }

  @RequiresApi(Build.VERSION_CODES.O)
  private fun createNotificationChannel(channelId: String, channelName: String): String {
    val chan = NotificationChannel(
      channelId,
      channelName, NotificationManager.IMPORTANCE_NONE
    )
    chan.description = "This is the description"
    chan.enableLights(true)
    chan.lightColor = Color.BLUE
    chan.lockscreenVisibility = Notification.VISIBILITY_PRIVATE
    val service = getSystemService(Context.NOTIFICATION_SERVICE) as NotificationManager
    service.createNotificationChannel(chan)
    return channelId
  }


  fun requestActivityUpdates() {
    val task = mActivityRecognitionClient.requestActivityUpdates(
      20000,
      mPendingIntent
    )

    task?.addOnSuccessListener {
      Log.i("ActivityRecogWrapper", "Starting activity updates")

    }

    task?.addOnCanceledListener {
      Log.i("ActivityRecogWrapper", "Activity Updates Cancelled")

    }
    task?.addOnCompleteListener {
      Log.i("ActivityRecogWrapper", "Activity Updates Started")

    }

    task?.addOnFailureListener {
      Toast.makeText(
        applicationContext,
        "Requesting activity updates failed to start",
        Toast.LENGTH_SHORT
      )
        .show()
    }
  }

  fun removeActivityUpdatesButtonHandler() {
    val task = mActivityRecognitionClient.removeActivityUpdates(
      mPendingIntent
    )
    task?.addOnSuccessListener {
      Log.i("ActivityRecogWrapper", "Stopping activity updates successfully")
      updatesRunning = false
    }

    task?.addOnFailureListener {
      Toast.makeText(
        applicationContext, "Failed to remove activity updates!",
        Toast.LENGTH_SHORT
      ).show()
    }
  }


  override fun onSensorChanged(event: SensorEvent) {
    val sensorType = event.sensor.type
    val currentTime = System.currentTimeMillis()
    val csvLine = "$currentTime,${event.values.joinToString(",")}\n"
    print(csvLine)
    csvFileHelper.writeCsvLine(sensorType, csvLine)
  }

  override fun onAccuracyChanged(sensor: Sensor, accuracy: Int) {
    // You can implement this method if needed
  }



  private fun activateSensors(folderStructure:List<String>, sensorInfoList: List<SensorInfo>) {
    sensorManager = getSystemService(Context.SENSOR_SERVICE) as SensorManager
    csvFileHelper = CsvFileHelper(this)

    sensorInfoList.forEach { sensorInfo ->
      val sensor = sensorManager.getDefaultSensor(sensorInfo.sensorType)
      sensor?.let {
        sensorManager.registerListener(this, it, sensorInfo.samplingRate)
        csvFileHelper.createCsvFile(sensorInfo.sensorName, sensorInfo.sensorType, folderStructure)
      }
    }
  }


  private fun stopSensors() {
    csvFileHelper.closeCsvFiles()
    sensorManager.unregisterListener(this)

  }

  override fun onDestroy() {
    super.onDestroy()
    running = false
    stopSensors()
    removeActivityUpdatesButtonHandler()
  }

  companion object {
    private val TAG = BackgroundDetectedActivitiesService::class.java.simpleName
    var mLocation: Location? = null
    var isServiceStarted = false
  }
}
