import android.annotation.SuppressLint
import android.hardware.Sensor
import java.io.Serializable

enum class SensorInfo(
  val sensorType: Int,
  var sensorName: String,
  var samplingRate: Int,
  val unitType: String
) : Serializable {
  ACC(
    Sensor.TYPE_LINEAR_ACCELERATION,
    "ACC",
    3,
    "t;x;y;z;a\n"
  ),
  ACG(
    Sensor.TYPE_ACCELEROMETER,
    "ACG",
    3,
    "t;x;y;z;a\n"
  ),
  AGG(
    Sensor.TYPE_GRAVITY,
    "AGG",
    3,
    "t;x;y;z;a\n",
  ),
  GYRO(
    Sensor.TYPE_GYROSCOPE,
    "GYRO",
    3,
    "t;x;y;z;a\n",
  ),
  HUM(
    Sensor.TYPE_RELATIVE_HUMIDITY,
    "HUM",
    1,
    "t;hum;a\n",
  ),
  MAGNET(
    Sensor.TYPE_MAGNETIC_FIELD,
    "MAGNET",
    3,
    "t;x;y;z;a\n",
  ),
  PROXIMITY(
    Sensor.TYPE_PROXIMITY,
    "PROXIMITY",
    1,
    "t;prox;a\n",
  ),
  ROTATION(
    Sensor.TYPE_ROTATION_VECTOR,
    "ROTATION",
    4,
    "t;x;y;z;0;a\n",
  ),
  PRESSURE(
    Sensor.TYPE_PRESSURE,
    "PRESSURE",
    1,
    "t;pressure;a\n",
  ),
  LIGHT(
    Sensor.TYPE_LIGHT,
    "LIGHT",
    1,
    "t;light;a\n",
  ),
  TEMP(
    Sensor.TYPE_AMBIENT_TEMPERATURE,
    "TEMP",
    1,
    "t;temp;a\n",
  ),
  STEP_COUNTER(
    Sensor.TYPE_STEP_COUNTER,
    "STEP_COUNTER",
    1,
    "t;steps;a\n",

    ),
  STEP_DETECTOR(
    Sensor.TYPE_STEP_DETECTOR,
    "STEP_DETECTOR",
    1,
    "t;steps;a\n",
    ),

  @SuppressLint("InlinedApi")

  HEART_RATE(
    Sensor.TYPE_HEART_RATE,
    "HEART_RATE",
    1,
    "t;bpm;a\n",
  ),
  ACC_WEAR(
    Sensor.TYPE_LINEAR_ACCELERATION,
    "ACC_WEAR",
    3,
    "t;x;y;z;a\n",
  ),
  ACG_WEAR(
    Sensor.TYPE_ACCELEROMETER,
    "ACG_WEAR",
    3,
    "t;x;y;z;a\n",
  ),
  GYRO_WEAR(
    Sensor.TYPE_GYROSCOPE,
    "GYRO_WEAR",
    3,
    "t;x;y;z;a\n",
  ),
  MAGNET_WEAR(
    Sensor.TYPE_MAGNETIC_FIELD,
    "MAGNET_WEAR",
    3,
    "t;x;y;z;a\n",
  ),
  @SuppressLint("InlinedApi")
  HEART_RATE_WEAR(
    Sensor.TYPE_HEART_RATE,
    "HEART_RATE_WEAR",
    1,
    "t;bpm;a\n",
  );

  companion object {

    fun getSensorById(id: Int): SensorInfo {
      for (sensorNeed: SensorInfo in values()) {
        if (sensorNeed.sensorType == id && "WEAR" !in sensorNeed.name) {
          return sensorNeed
        }
      }
      return ACG
    }

    fun getSensorByName(name: String): SensorInfo {
      for (sensorNeed: SensorInfo in values()) {
        if (sensorNeed.sensorName == name && "WEAR" !in sensorNeed.name) {
          return sensorNeed
        }
      }
      return ACG
    }
  }

}

